export default {
  props: ['fieldModel', 'componentFieldSettings', 'fieldName'],
  data() {
    const {
      allowed_providers: allowedProviders,
      vimeo_provider_details: vimeoDescription,
      youtube_provider_details: youtubeDescription,
      width,
      height
    } = this.componentFieldSettings.settings;

    return {
      allowedProviders,
      vimeoDescription,
      youtubeDescription,
      width,
      height,
      hasErrors: false
    }
  },
  methods: {
    validateUrl(event) {
      this.hasErrors = false;
      const url = this.fieldModel.url;
      let embedUrl = '';

      const regexPatterns = {
        vimeo: /^https?:\/\/(www\.)?vimeo.com\/(channels\/[a-zA-Z0-9]*\/)?(?<id>[0-9]*)(\/[a-zA-Z0-9]+)?(\#t=(\d+)s)?$/,
        youtube: /^https?:\/\/(www\.)?((?!.*list=)youtube\.com\/watch\?.*v=|youtu\.be\/)(?<id>[0-9A-Za-z_-]*)/
      };

      const match = url.match(regexPatterns[this.fieldModel.provider]);

      if (match) {
        const videoId = match.groups.id;

        if (this.fieldModel.provider === 'vimeo') {
          embedUrl = `https://player.vimeo.com/video/${videoId}`;
          this.fieldModel.value = `<iframe src="${embedUrl}" width="${this.width}" height="${this.height}" frameborder="0" allow="autoplay; fullscreen; picture-in-picture" target_type="vimeo" target_id="${videoId}" allowfullscreen></iframe>`;
        }
        else if (this.fieldModel.provider === 'youtube') {
          embedUrl = `https://www.youtube.com/embed/${videoId}`;
          this.fieldModel.value = `<iframe width="${this.width}" height="${this.height}" src="${embedUrl}" title="YouTube video player" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" target_type="youtube" target_id="${videoId}" allowfullscreen></iframe>`;
        }
      } else {
        this.hasErrors = true;
      }
    },
  },
  computed: {
    hasAllowedProviders() {
      return Object.keys(this.allowedProviders).length > 0;
    },
  },
  template: `
    <fieldset class="fieldset form-item">
      <legend class="fieldset__legend fieldset__legend--visible">
        <span v-if="!componentFieldSettings.hide_label" class="fieldset__label">{{ componentFieldSettings.label }}</span>
      </legend>
      <div class="castorcito-container">
        <div v-if="hasAllowedProviders" class="allowed-provider">
          <div class="form-item form-type--select">
            <label :for="fieldName + '_allowed_providers_select'" class="form-item__label">Allowed providers</label>
            <select
              class="form-select form-element form-element--type-select"
              v-model="fieldModel.provider"
              :id="fieldName + '_allowed_providers_select'"
              :name="fieldName + '_allowed_providers_select'">
              <option value="iframe">Iframe</option>
              <option v-for="(child, i) in allowedProviders" :key="i" :value="i">{{ child }}</option>
            </select>
          </div>
          <div v-if="fieldModel.provider != 'iframe'" class="form-item form-type--textfield">
            <div v-if="hasErrors" role="contentinfo" class="messages-list__item messages messages--error">
              <div role="alert">
                <div class="messages__header">
                  <h2 :id="fieldName + 'message-error-url'" class="messages__title">Error message</h2>
                </div>
                <div class="messages__content">
                  <ul class="messages__list">
                    <li class="messages__item">Invalid video URL.</li>
                  </ul>
                </div>
              </div>
            </div>
            <label :for="fieldName + '_allowed_providers_input'" class="form-item__label" :class="hasErrors ? 'has-error' : ''">{{ ' Insert a ' + fieldModel.provider + ' URL' }}</label>
            <input :id="fieldName + '_allowed_providers_input'" :name="fieldName + '_allowed_providers_input'" v-model="fieldModel.url" @blur="validateUrl" type="text" size="60" maxlength="255" class="form-text form-element" :class="hasErrors ? 'error' : ''" />
            <div class="form-item__description has-error" v-html="fieldModel.provider == 'vimeo' ? vimeoDescription : youtubeDescription"></div>
          </div>
        </div>
        <div v-if="!hasAllowedProviders || fieldModel.provider == 'iframe'" class="form-textarea-wrapper">
          <label :for="fieldName" class="form-item__label">Iframe</label>
          <textarea
            class="text-full form-textarea resize-vertical form-element form-element--type-textarea form-element--api-textarea"
            :data-drupal-selector="fieldName"
            :id="fieldName"
            :name="fieldName"
            :ref="fieldName"
            rows="5"
            cols="60"
            placeholder=""
            v-model="fieldModel.value">
          </textarea>
          <div class="form-item__description" v-html="componentFieldSettings.description"></div>
        </div>
      </div>
    </fieldset>
  `
};
