<?php

namespace Drupal\castorcito;

use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\Site\Settings;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Url;

 /*
 * @package Drupal\CastorcitoManager
 */
class CastorcitoManager {

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The block manager.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected $blockManager;

  /**
   * Constructs a CastorcitoManager.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param Drupal\jfu\Services\EntityTypeManagerInterface $entity_type_manager
   *   The The entity type manager.
   * @param \Drupal\Core\Block\BlockManagerInterface $block_manager
   *   The block manager.
   * @param \Drupal\Core\File\FileUrlGenerator; $file_url_generator
   *   The file url generator.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager, EntityTypeManagerInterface $entity_type_manager, BlockManagerInterface $block_manager, FileUrlGenerator $file_url_generator) {
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->blockManager = $block_manager;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
  *  List all components as options.
  * @return: Array.
  */
  public function castorcitoComponentsOptions(): array {
    $options = [];
    $components = $this->castorcitoComponents();
    foreach ($components as $component) {
      if (!$component->get('inside_container')) {
        $options[$component->id()] = $component->label();
      }
    }

    return $options;
  }

  /**
   * List all components.
   * @param array $ids
   * @return array
   */
  public function castorcitoComponents(array $ids = NULL): array {
    $components = $this->entityTypeManager->getStorage('castorcito_component')->loadMultiple($ids);

    return $components;
  }

  /**
   * Retrieves a list of components with their field settings.
   *
   * @param array $ids
   *   An array of component IDs to fetch.
   *
   * @return array
   *   An associative array containing:
   *   - 'components': An array of component models.
   *   - 'components_fields_settings': An array of field configuration settings.
   */
  public function getCastorcitoComponents(array $ids): array {
    $components = [];
    $components_fields_settings = [];

    if ($all_components = $this->castorcitoComponents($ids)) {
      foreach ($all_components as $key => $component) {
        $components[$key] = $component->getModel();
        $components_fields_settings[$key]['fields'] = $component->get('field_configuration');
        $components_fields_settings[$key]['order_fields'] = array_keys($components[$key]['fields']);
        $components_fields_settings[$key]['settings']['enable_edit_name'] = $component->get('enable_edit_name');
      }
    }

    $allowed_fields = [
      'container',
      'advanced_container'
    ];

    if (!empty($components_fields_settings)) {
      foreach ($components_fields_settings as $fields_settings) {
        foreach ($fields_settings['fields'] as $field) {
          if (in_array($field['id'], $allowed_fields)) {
            if (!empty($field['settings']) && isset($field['settings']['allowed_children']) && !empty($field['settings']['allowed_children'])) {
              $child_components = $this->getCastorcitoComponents(array_keys($field['settings']['allowed_children']));
              $components = array_merge($components, $child_components['components']);
              $components_fields_settings = array_merge($components_fields_settings, $child_components['components_fields_settings']);
            }
          }
        }
      }
    }

    return [
      'components' => $components,
      'components_fields_settings' => $components_fields_settings
    ];
  }

  /**
   * Selected components.
   * 
   * @param array $components config
   * @return array
   */
  public function selectedComponents(array $components): array {
    $selected_components = [];
    foreach ($components as $component) {
      if ($component !== 0) {
        array_push($selected_components, $component);
      }
    }

    return $selected_components;
  }

  /**
   * Build castorcito widget name.
   * 
   * @param array $params
   * @return string
   */
  public function configWidgetName(array $params): string {
    return 'castorcito.field.' . $params['entity_type'] . '.' . $params['bundle'] . '.' . $params['field_name'];
  }

  /**
   * Retrieves all block definitions available in the system.
   *
   * This function returns an array of block definitions, which includes
   * metadata for each block, such as its plugin ID, label, provider, and more.
   *
   * @return array
   *   An associative array of block definitions, keyed by plugin ID.
   */
  public function getBlockDefinitions() {
    return $this->blockManager->getDefinitions();
  }

  /**
   * {@inheritdoc}
   */
  public function entityAutocompleteUrl($target_type, $selection_handler = 'default', $selection_settings = []): string {
    $data = serialize($selection_settings) . $target_type . $selection_handler;
    $selection_settings_key = Crypt::hmacBase64($data, Settings::getHashSalt());

    $key_value_storage = \Drupal::keyValue('entity_autocomplete');
    if (!$key_value_storage->has($selection_settings_key)) {
      $key_value_storage->set($selection_settings_key, $selection_settings);
    }

    $routes_parameters = [
      'target_type' => $target_type,
      'selection_handler' => $selection_handler,
      'selection_settings_key' => $selection_settings_key,
    ];

    $url = Url::fromRoute('system.entity_autocomplete', $routes_parameters)->toString(TRUE);

    return $url->getGeneratedUrl();
  }

  /**
   * {@inheritdoc}
   */
  public function generalUrls(): array {
    $urls['url_token'] = Url::fromRoute('system.csrftoken')->toString(TRUE)->getGeneratedUrl();
    $urls['url_public'] = $this->fileUrlGenerator->generateAbsoluteString('public://');
    $urls['url_upload_image'] = Url::fromRoute('rest.castorcito_ajax_upload_image_resource.POST')->toString(TRUE)->getGeneratedUrl();

    return $urls;
  }

}
