<?php

namespace Drupal\castorcito\Form;

use Drupal\castorcito\CastorcitoComponentFieldManager;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for castorcito component edit form.
 *
 * @internal
 */
class CastorcitoComponentEditForm extends CastorcitoComponentFormBase {

  /**
   * The castorcito component field manager service.
   *
   * @var \Drupal\castorcito\CastorcitoComponentFieldManager
   */
  protected $componentFieldManager;

  /**
   * Constructs an CastorcitoComponentEditForm object.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $component_storage
   *   The entity storage.
   * @param \Drupal\castorcito\CastorcitoComponentFieldManager $component_field_manager
   *   The castorcito component field manager service.
   */
  public function __construct(EntityStorageInterface $component_storage, CastorcitoComponentFieldManager $component_field_manager) {
    parent::__construct($component_storage);
    $this->componentFieldManager = $component_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')->getStorage('castorcito_component'),
      $container->get('plugin.manager.castorcito_component_field')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $cfm = $this->componentFieldManager->getDefinitions();
    $model = $this->entity->getModel();
    $new_field_options = [];

    if (!empty($cfm)) {
      foreach ($cfm as $field => $definition) {
        $new_field_options[$field] = $definition['label'];
      }
    }

    $form['#title'] = $this->t('Edit %label component', [
      '%label' => $this->entity->label(),
    ]);

    $form['#attached']['library'][] = 'castorcito/castorcito.admin';
    $form['components_extra_config'] = [
      '#type' => 'details',
      '#title' => $this->t('Extra configuration'),
    ];

    $form['components_extra_config']['allow_classes'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allows you to add classes to the components container'),
      '#default_value' => isset($model['options']) ? $model['options']['allow_classes'] : '',
    ];

    $form['components_extra_config']['allow_select_class'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allows defining custom classes for the component container'),
      '#default_value' => isset($model['options']) ? $model['options']['allow_select_class'] : '',
    ];

    $form['components_extra_config']['class_list'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Add custom classes'),
      '#description' => $this->t('Add classes separated by "," and without spaces. For example: class1,class-2,class_3.'),
      '#default_value' => isset($model['options']) ? $model['options']['class_list'] : '',
      '#states' => [
        'visible' => [
          ':input[name="allow_select_class"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['components_extra_config']['allow_group_items'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allows you to add group items to the components'),
      '#default_value' => isset($model['options']) ? $model['options']['allow_group_items'] : '',
    ];

    $form['components_extra_config']['full_width'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allows you to activate the full width of the component container.'),
      '#default_value' => isset($model['options']) ? $model['options']['full_width'] : '',
    ];

    $form['components_field'] = [
      '#type' => 'details',
      '#title' => $this->t('Fields'),
      '#open' => TRUE,
    ];

    $form['components_field']['component_item'] = [
      '#type' => 'item',
      '#title' => 'Add field',
      '#description' => $this->t('Shows all the fields defined in the Castorcito module plugin, if you want to add more fields you can do it by creating your own custom module.'),
    ];

    $form['components_field']['component_item']['new_field'] = [
      '#type' => 'select',
      '#title' => '',
      '#options' => $new_field_options,
      '#empty_option' => $this->t('- Select a field type -'),
      '#prefix' => '<div class="cc-field-new clearfix">',
    ];

    $form['components_field']['component_item']['new_add'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add'),
      '#validate' => ['::componentFieldValidate'],
      '#submit' => ['::componentFieldConfiguration'],
      '#suffix' => '</div>',
    ];

    $form['components_field']['component_field_table'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Label'),
        $this->t('Machine name'),
        $this->t('Field type'),
        $this->t('Weight'),
        $this->t('Operations'),
      ],
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'component-field-order-weight',
        ],
      ],
      '#empty' => $this->t('There are currently no fields for the component. Add by selecting a field in the "Add field" option.'),
      '#weight' => 5,
    ];

    if (!empty($model['fields'])) {
      $number = 0;

      foreach ($model['fields'] as $key => $value) {
        $field = $this->entity->getComponentField($key);
        $form['components_field']['component_field_table'][$key]['#attributes']['class'][] = 'draggable';
        $form['components_field']['component_field_table'][$key]['label'] = [
          '#tree' => FALSE,
          'data' => [
            'label' => [
              '#plain_text' => $field->getFieldLabel(),
            ],
          ],
        ];

        $form['components_field']['component_field_table'][$key]['machine_name'] = [
          '#tree' => FALSE,
          'data' => [
            'label' => [
              '#plain_text' => $field->getFieldName(),
            ],
          ],
        ];


        $form['components_field']['component_field_table'][$key]['field_type'] = [
          '#tree' => FALSE,
          'data' => [
            'label' => [
              '#plain_text' => $field->getPluginId(),
            ],
          ],
        ];

        $form['components_field']['component_field_table'][$key]['weight'] = [
          '#type' => 'weight',
          '#title' => $this->t('Weight for'),
          '#title_display' => 'invisible',
          '#default_value' => $number,
          '#attributes' => [
            'class' => ['component-field-order-weight'],
          ],
        ];

        $operations['edit'] = [
          'title' => $this->t('Edit'),
          'url' => Url::fromRoute('castorcito.component_field_edit_form', [
            'castorcito_component' => $this->entity->id(),
            'field_type' => $key,
          ]),
        ];

        $operations['delete'] = [
          'title' => $this->t('Delete'),
          'url' => Url::fromRoute('castorcito.component_field_delete_form', [
            'castorcito_component' => $this->entity->id(),
            'field_type' => $key,
          ]),
        ];


        $form['components_field']['component_field_table'][$key]['operations'] = [
          '#type' => 'operations',
          '#links' => $operations,
        ];

        $number ++;
      }
    }

    return $form;
  }

  /**
   * Validate handler for component field.
   */
  public function componentFieldValidate($form, FormStateInterface $form_state) {
    if (!$form_state->getValue('new_field')) {
      $form_state->setErrorByName('new_field', $this->t('Select the field to add.'));
    }
  }

  /**
   * Submit handler for component field.
   */
  public function componentFieldConfiguration($form, FormStateInterface $form_state) {

    $form_state->setRedirect(
      'castorcito.component_field_add_form',
      [
        'castorcito_component' => $this->entity->id(),
        'field_type' => $form_state->getValue('new_field'),
      ],
    );
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $options = [
      'allow_classes' => $form_state->getValue('allow_classes'),
      'allow_select_class' => $form_state->getValue('allow_select_class'),
      'class_list' => $form_state->getValue('class_list'),
      'allow_group_items' => $form_state->getValue('allow_group_items'),
      'full_width' => $form_state->getValue('full_width'),
    ];

    $this->entity->setBaseModel($form_state->getValue('label'));
    $this->entity->setModelOptions($options);

    if (!empty($form_state->getValue('component_field_table'))) {
      $this->orderModel($form_state->getValue('component_field_table'));
    }

    parent::save($form, $form_state);
    $this->messenger()->addStatus($this->t('Changes to the component have been saved.'));
  }

  /**
   * {@inheritdoc}
   */
  private function orderModel($order) {
    $model = $this->entity->getModel();
    $model['fields'] = array_merge($order, $model['fields']);
    $this->entity->setModel($model);
  }
}
