<?php

namespace Drupal\castorcito\Plugin\CastorcitoComponentField;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\Form\FormStateInterface;
use Drupal\castorcito\ConfigurableComponentFieldBase;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;

/**
 * Entity reference configuration.
 *
 * @CastorcitoComponentField(
 *   id = "entity_reference",
 *   label = @Translation("Entity reference"),
 *   description = @Translation("Field to reference other content.")
 * )
 */
class EntityReference extends ConfigurableComponentFieldBase {

  /**
   * {@inheritdoc}
   */
  public function defaultModel() {
    return [
      'entity_type' => '',
      'entity_id' => '',
      'value' => ''
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'entity_type' => '',
      'entity_bundle' => [],
      'link_autocomplete' => ''
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $entity_type_manager = \Drupal::entityTypeManager();
    $filter = function (string $entity_type_id) use ($entity_type_manager): bool {
      return $entity_type_manager->getDefinition($entity_type_id)
        ->hasKey('id');
    };

    $entity_type_labels = \Drupal::service('entity_type.repository')->getEntityTypeLabels(TRUE);
    $first_group = reset($entity_type_labels);
    $options = array_filter($first_group, $filter, ARRAY_FILTER_USE_KEY);

    $form['entity_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity type'),
      '#default_value' => $this->configuration['entity_type'],
      '#required' => TRUE,
      '#options' => $options,
      '#empty_option' => $this->t('- Select -'),
    ];

    if (!empty($options)) {
      foreach ($options as $key => $option) {
        if ($this->isBundleEntityType($key)) {
          $bundle_info = \Drupal::service('entity_type.bundle.info');
          $types = $bundle_info->getBundleInfo($key);

          if (!empty($types)) {
            $bundle_options = [];
            foreach ($types as $type_id => $info) {
              $bundle_options[$type_id] = $info['label'];
            }

            $form[$key . '_bundle'] = [
              '#type' => 'checkboxes',
              '#title' => $this->t('Entity bundle'),
              '#options' => $bundle_options,
              '#default_value' => $this->configuration['entity_bundle'],
              '#description' => $this->t('Choose the required bundle types.'),
              '#multiple' => TRUE,
              '#states' => [
                'visible' => [
                  ':input[name="settings[entity_type]"]' => ['value' => $key],
                ],
              ],
            ];
          }
        }
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $entity_type = $form_state->getValue('entity_type');

    if ($this->isBundleEntityType($entity_type)) {
      if (empty(array_filter($form_state->getValue($entity_type . '_bundle')))) {
        $form_state->setErrorByName($entity_type . '_bundle', $this->t('Entity bundle field is required.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $entity_type = $form_state->getValue('entity_type');
    $this->configuration['entity_type'] = $entity_type;

    $selection_settings['auto_create'] = '';
    $selection_settings['match_operator'] = 'CONTAINS';
    $selection_settings['match_limit'] = 10;
    $selection_settings['target_bundles'] = '';
    $selection_settings['sort'] = [
      'field' => '_none',
      'direction' => 'ASC'
    ];

    if ($this->isBundleEntityType($entity_type)) {
      $target_bundles = array_filter($form_state->getValue($entity_type . '_bundle'));
      $this->configuration['entity_bundle'] = $target_bundles;
      $selection_settings['target_bundles'] = $target_bundles;
      $selection_settings['auto_create_bundle'] = reset($target_bundles);
    }

    $castorcito_manager = \Drupal::service('castorcito.manager');
    $selection_handler = 'default:' . $entity_type;
    $this->configuration['link_autocomplete'] = $castorcito_manager->entityAutocompleteUrl($entity_type, $selection_handler, $selection_settings);
  }

  /**
   * {@inheritdoc}
   */
  private function isBundleEntityType($entity_type) {
    $entity_type_manager = \Drupal::entityTypeManager();
    $entity_type_definition = $entity_type_manager->getDefinition($entity_type);

    return $entity_type_definition->getBundleEntityType();
  }

  /**
   * {@inheritdoc}
   */
  private function entityAutocompleteUrl($entity_type, $selection_settings) {
    $data = serialize($selection_settings) . $entity_type . 'default:' . $entity_type;
    $selection_settings_key = Crypt::hmacBase64($data, Settings::getHashSalt());

    $key_value_storage = \Drupal::keyValue('entity_autocomplete');
    if (!$key_value_storage->has($selection_settings_key)) {
      $key_value_storage->set($selection_settings_key, $selection_settings);
    }

    $routes_parameters = [
      'target_type' => $entity_type,
      'selection_handler' => 'default:' . $entity_type,
      'selection_settings_key' => $selection_settings_key,
    ];

    $url = Url::fromRoute('system.entity_autocomplete', $routes_parameters)->toString(TRUE);

    return $url->getGeneratedUrl();
  }

}
