<?php

namespace Drupal\castorcito\Plugin\CastorcitoComponentField;

use Drupal\Core\Form\FormStateInterface;
use Drupal\castorcito\ConfigurableComponentFieldBase;

/**
 * List Text configuration.
 *
 * @CastorcitoComponentField(
 *   id = "list_text",
 *   label = @Translation("List text"),
 *   description = @Translation("Configuration for the list text field.")
 * )
 */
class ListText extends ConfigurableComponentFieldBase {

  /**
   * {@inheritdoc}
   */
  public function defaultModel() {
    return [
      'value' => '',
    ];
  }


  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'options' => [
        [
          'key' => '',
          'label' => '',
        ]
      ],
      'condition' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    if (!$form_state->get('items_count')) {
      $form_state->set('items_count', max(count($this->configuration['options']), 0) - 1);
    }

    if (!$form_state->get('removed_item')) {
      $form_state->set('removed_item', []);
    }


    $form['#tree'] = TRUE;
    $form['field_table_fieldset'] = [
      '#type' => 'details',
      '#title' => $this->t('Add options for the field'),
      '#prefix' => '<div id="names-fieldset-wrapper">',
      '#suffix' => '</div>',
      '#open' => TRUE,
    ];

    $form['field_table_fieldset']['condition'] = [
      '#type' => 'checkbox',
      '#title' => t('Use as condition field'),
      '#default_value' => $this->configuration['condition'],
      '#description' => '<p>' . $this->t('Use this option to make the field work as a conditional field.') . '</p>',
    ];

    $form['field_table_fieldset']['table'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Allowed values'),
        $this->t('Delete'),
        $this->t('Weight'),
      ],
      '#attributes' => [
        'id' => 'allowed-values-order',
        'data-field-list-table' => TRUE,
        'class' => ['allowed-values-table'],
      ],
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'weight',
        ],
      ],
    ];

    $max = $form_state->get('items_count');
    $removed_item = $form_state->get('removed_item');
    for ($i = 0; $i <= $max; $i++) {
      if (!in_array($i, $removed_item)) {
        $form['field_table_fieldset']['table'][$i] = [
          '#attributes' => [
            'class' => ['draggable'],
          ],
          '#weight' => $i,
        ];

        $form['field_table_fieldset']['table'][$i]['item'] = [
          'label' => [
            '#type' => 'textfield',
            '#maxlength' => 255,
            '#title' => $this->t('Name'),
            '#default_value' => $this->configuration['options'][$i]['label'],
            '#weight' => -1,
            '#required' => TRUE,
          ],
          'key' => [
            '#type' => 'textfield',
            '#title' => $this->t('Value'),
            '#weight' => 0,
            '#default_value' => $this->configuration['options'][$i]['key'],
            '#required' => TRUE,
          ],
          'state' => [
            '#type' => 'select',
            '#title' => $this->t('Form state'),
            '#default_value' => isset($this->configuration['conditions']) ? $this->configuration['conditions'][$i]['state'] : '',
            '#options' => [
              'visible' => $this->t('Visible'),
              'invisible' => $this->t('Invisible'),
            ],
            '#states'   => [
              'visible' => [
                ':input[name="settings[field_table_fieldset][condition]"]' => ['checked'   => TRUE],
              ],
            ]
            ],
          'fields' => [
            '#type' => 'checkboxes',
            '#title' => $this->t('Fields'),
            '#options' => $this->listFields(),
            '#default_value' => isset($this->configuration['conditions']) ? $this->configuration['conditions'][$i]['fields'] : [],
            '#description' => $this->t('If this option is empty, you must add at least one field for conditioning.'),
            '#multiple' => TRUE,
            '#states'   => [
              'visible' => [
                ':input[name="settings[field_table_fieldset][condition]"]' => ['checked'   => TRUE],
              ],
            ]
            ],
        ];

        $form['field_table_fieldset']['table'][$i]['delete'] = [
          '#type' => 'submit',
          '#value' => $this->t('Remove'),
          '#name' => $i,
          '#submit' => [
            [$this, 'removeCallback']
          ],
          '#limit_validation_errors' => [],
          '#ajax' => [
            'callback' => [$this, 'addmoreCallback'],
            'wrapper' => 'names-fieldset-wrapper',
            'effect' => 'fade',
          ],
        ];
        $form['field_table_fieldset']['table'][$i]['weight'] = [
          '#type' => 'weight',
          '#title' => $this->t('Weight for row @number', ['@number' => $i + 1]),
          '#title_display' => 'invisible',
          '#delta' => 50,
          '#default_value' => $i,
          '#attributes' => ['class' => ['weight']],
        ];
      }
    }

    $form['field_table_fieldset']['actions'] = [
      '#type' => 'actions',
    ];
    $form['field_table_fieldset']['actions']['add_name'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add one more'),
      '#submit' => [
       [$this, 'addMoreSubmit'],
      ],
      '#ajax' => [
        'callback' => [$this, 'addmoreCallback'],
        'wrapper' => 'names-fieldset-wrapper',
      ],
    ];

   return $form;
 }

 /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $field_table_fieldset = $form_state->getValue('field_table_fieldset');

    if (!empty($field_table_fieldset)) {
      $options = [];
      $conditions = [];
      $item = 0;

      foreach ($field_table_fieldset['table'] as $key => $value) {
        $options[$item]['key'] = $value['item']['key'];
        $options[$item]['label'] = $value['item']['label'];
        $conditions[$item]['state'] = $value['item']['state'];
        $conditions[$item]['fields'] = array_filter($value['item']['fields']);
        $item ++;
      }

      $this->configuration['options'] = $options;
      $this->configuration['condition'] = $field_table_fieldset['condition'];

      if ($field_table_fieldset['condition']) {
        $this->configuration['conditions'] = $conditions;
      }
      else {
        if (isset($this->configuration['conditions'])) {
          unset($this->configuration['conditions']);
        }
      }

      if (!empty($this->getFieldName())) {
        $this->updateConfiguration();
      }
    }
  }

 /**
  * Ajax callback for the "Add another item" button
  */
  public function addmoreCallback(array &$form, FormStateInterface $form_state) {
    return $form['settings']['field_table_fieldset'];
  }

  /**
   * Adds a new option.
   *
   * @param array $form
   *   The form array to add elements to.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function addMoreSubmit(array &$form, FormStateInterface $form_state) {
    $items_count = $form_state->get('items_count');
    $add_button = $items_count + 1;
    $form_state->set('items_count', $add_button);
    $form_state->setRebuild();
  }

  /**
   * Deletes a row/option.
   *
   * @param array $form
   *   The form array to add elements to.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function removeCallback(array &$form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    $index_remove = $trigger['#name'];
    $removed_fields = $form_state->get('removed_item');
    $removed_fields[] = $index_remove;
    $form_state->set('removed_item', $removed_fields);
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  protected function listFields() {
    $options = [];

    foreach ($this->component->getComponentFields() as $key => $field) {
      if (!empty($this->getFieldName()) && $field->getFieldName() != $this->getFieldName()) {
        $options[$field->getFieldName()] = $field->getFieldLabel();
      }

      if (empty($this->getFieldName())) {
        $options[$field->getFieldName()] = $field->getFieldLabel();
      }
    }

    return $options;
  }
}
