<?php

namespace Drupal\castorcito\Plugin\Field\FieldWidget;

use Drupal\Core\Config\ConfigFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Render\ElementInfoManagerInterface;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\json_field\Plugin\Field\FieldWidget\JsonTextareaWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\castorcito\CastorcitoManager;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;


/**
 * Plugin implementation of the 'castorcito_component_widget' widget.
 *
 * @FieldWidget(
 *   id = "castorcito_component_widget",
 *   module = "castorcito",
 *   label = @Translation("Castorcito component"),
 *   description = @Translation("Define all components."),
 *   field_types = {
 *     "json",
 *     "json_native",
 *     "json_native_binary"
 *   },
 * )
 */

class CastorcitoComponentWidget extends JsonTextareaWidget {

  /**
   * The castorcito manager.
   *
   * @var Drupal\castorcito\CastorcitoManager;
   */
  protected $castorcitoManager;

  /**
   * The config widget name.
   *
   * @var Drupal\castorcito\Plugin\Field\FieldWidget
   */
  protected $configWidgetName;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    ElementInfoManagerInterface $element_info,
    ExtensionPathResolver $extension_path_resolver,
    ConfigFactory $configFactory,
    CastorcitoManager $castorcito_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->elementInfo = $element_info;
    $this->extensionPathResolver = $extension_path_resolver;
    $this->configFactory = $configFactory;
    $this->castorcitoManager = $castorcito_manager;
    $params = [
      'entity_type' => $this->fieldDefinition->get('entity_type'),
      'bundle' => $this->fieldDefinition->get('bundle'),
      'field_name' => $this->fieldDefinition->getName(),
    ];
    $this->configWidgetName = $this->castorcitoManager->configWidgetName($params);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('element_info'),
      $container->get('extension.path.resolver'),
      $container->get('config.factory'),
      $container->get('castorcito.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    if (!isset($form['#is_castorcito'])) {
      $form['#is_castorcito'] = 'field--widget-castorcito-component-widget';
    }

    $element['value'] = $element + [
      '#type' => 'textarea',
      '#default_value' => $items[$delta]->value,
      '#attributes' => [
        'target-weight' => $delta,
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $url_widget_config = Url::fromRoute('castorcito.widget_config')->setRouteParameters([
      'entity_type' => $this->fieldDefinition->get('entity_type'),
      'bundle' => $this->fieldDefinition->get('bundle'),
      'field_name' => $this->fieldDefinition->getName(),
    ]);

    $element['components_link'] = [
      '#type' => 'link',
      '#title' => $this->t('Edit configurations'),
      '#url' => $url_widget_config,
      '#attributes' => [
        'class' => ['use-ajax'],
        'data-dialog-options' => Json::encode(
          [
            'width' => '800',
            'clasess' => ['ui-dialog'],
          ]
        ),
        'data-dialog-type' => 'modal',
      ],
      '#prefix' => '<div>',
      '#suffix' => '</div>',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $widget_config = $this->configFactory->getEditable($this->configWidgetName);
    $components_options = $this->castorcitoManager->castorcitoComponentsOptions();

    $summary[] = $this->t('You don\'t have any selected components for this field.');
    $summary[] = $this->t('Choose the components you want to use.');
      
    if (!empty($widget_config->get('components'))) {
      $items = [];
      foreach ($widget_config->get('components') as $component) {
        if ($component !== 0) {
          $items[] = $components_options[$component];
        }
      }

      $summary = [];
      $summary[] = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        '#title' => $this->t('Allowed components'),
        '#items' => $items,
      ];
    }

    return $summary;
  }

}
